#!/usr/bin/env python3
"""
Windows Disk Manager (System Management Tool)

Parameters:
    --list: List all disk drives
    --info DRIVE: Show detailed drive information
    --format DRIVE: Format specified drive
    --mount DRIVE LETTER: Mount drive to specified letter
    --unmount DRIVE: Unmount specified drive
    --smart DRIVE: Show S.M.A.R.T. information
    --format FORMAT: Output format (table/json)

Requires: wmi, win32api, tabulate packages and admin privileges
Warning: Use extreme caution with format operations - data will be permanently lost
"""

import os
import sys
import json
import time
import logging
import subprocess
import win32api
import win32file
import wmi
from typing import List, Dict, Optional, Union
from datetime import datetime
from pathlib import Path

from tabulate import tabulate

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

class DiskManager:
    """Manages Windows disk drives and storage operations."""
    
    def __init__(self):
        """Initialize the DiskManager."""
        self.wmi = wmi.WMI()
    
    def get_disk_drives(self) -> List[Dict]:
        """
        Get information about physical disk drives.
        
        Returns:
            List[Dict]: List of disk drive information
        """
        drives = []
        
        try:
            for disk in self.wmi.Win32_DiskDrive():
                drive_info = {
                    "name": disk.Caption,
                    "device_id": disk.DeviceID,
                    "interface_type": disk.InterfaceType,
                    "media_type": disk.MediaType,
                    "model": disk.Model,
                    "size": disk.Size,
                    "partitions": disk.Partitions,
                    "status": disk.Status,
                    "health": self._check_disk_health(disk.DeviceID)
                }
                drives.append(drive_info)
                
        except Exception as e:
            logger.error(f"Error getting disk drives: {e}")
            
        return drives
    
    def get_volumes(self) -> List[Dict]:
        """
        Get information about disk volumes (logical drives).
        
        Returns:
            List[Dict]: List of volume information
        """
        volumes = []
        
        try:
            for volume in self.wmi.Win32_LogicalDisk():
                volume_info = {
                    "name": volume.Caption,
                    "description": volume.Description,
                    "drive_type": self._get_drive_type(volume.DriveType),
                    "filesystem": volume.FileSystem,
                    "free_space": volume.FreeSpace,
                    "size": volume.Size,
                    "volume_name": volume.VolumeName,
                    "usage_percent": self._calculate_usage_percent(
                        volume.Size, volume.FreeSpace
                    )
                }
                volumes.append(volume_info)
                
        except Exception as e:
            logger.error(f"Error getting volumes: {e}")
            
        return volumes
    
    def get_disk_usage(self, path: str) -> Dict:
        """
        Get disk usage information for a specific path.
        
        Args:
            path: Path to check disk usage
            
        Returns:
            Dict: Disk usage information
        """
        try:
            total, used, free = win32api.GetDiskFreeSpaceEx(path)
            
            return {
                "path": path,
                "total": total,
                "used": used,
                "free": free,
                "usage_percent": self._calculate_usage_percent(total, free)
            }
            
        except Exception as e:
            logger.error(f"Error getting disk usage for {path}: {e}")
            return {
                "error": str(e),
                "path": path
            }
    
    def check_disk(self, drive: str) -> Dict:
        """
        Run disk check on specified drive.
        
        Args:
            drive: Drive letter to check (e.g., "C:")
            
        Returns:
            Dict: Check disk results
        """
        try:
            # Run chkdsk in read-only mode
            cmd = ["chkdsk", drive, "/f"]
            result = subprocess.run(cmd, capture_output=True, text=True)
            
            return {
                "drive": drive,
                "success": result.returncode == 0,
                "output": result.stdout,
                "error": result.stderr if result.stderr else None
            }
            
        except Exception as e:
            logger.error(f"Error checking disk {drive}: {e}")
            return {
                "drive": drive,
                "success": False,
                "error": str(e)
            }
    
    def cleanup_disk(self, drive: str) -> Dict:
        """
        Run disk cleanup on specified drive.
        
        Args:
            drive: Drive letter to clean (e.g., "C:")
            
        Returns:
            Dict: Cleanup results
        """
        try:
            # Run cleanmgr
            cmd = ["cleanmgr", "/d", drive]
            result = subprocess.run(cmd, capture_output=True, text=True)
            
            return {
                "drive": drive,
                "success": result.returncode == 0,
                "output": result.stdout,
                "error": result.stderr if result.stderr else None
            }
            
        except Exception as e:
            logger.error(f"Error cleaning disk {drive}: {e}")
            return {
                "drive": drive,
                "success": False,
                "error": str(e)
            }
    
    def monitor_disk_usage(self, path: str, interval: int = 60,
                          threshold: float = 90.0) -> None:
        """
        Monitor disk usage and alert on high usage.
        
        Args:
            path: Path to monitor
            interval: Check interval in seconds
            threshold: Usage threshold percentage for alerts
        """
        logger.info(f"Starting disk usage monitoring for {path}")
        
        try:
            while True:
                usage = self.get_disk_usage(path)
                
                if "error" in usage:
                    logger.error(usage["error"])
                    break
                    
                if usage["usage_percent"] >= threshold:
                    logger.warning(
                        f"High disk usage on {path}: "
                        f"{usage['usage_percent']}%"
                    )
                    
                logger.info(
                    f"Disk usage on {path}: {usage['usage_percent']}% "
                    f"(Free: {self._format_size(usage['free'])})"
                )
                
                time.sleep(interval)
                
        except KeyboardInterrupt:
            logger.info("Disk usage monitoring stopped")
    
    def _check_disk_health(self, device_id: str) -> str:
        """
        Check disk health using SMART data.
        
        Args:
            device_id: Disk device ID
            
        Returns:
            str: Disk health status
        """
        try:
            # This is a simplified check. In a real implementation,
            # you would use a library like pySMART to get detailed SMART data
            for disk in self.wmi.Win32_DiskDrive():
                if disk.DeviceID == device_id:
                    if disk.Status == "OK":
                        return "Healthy"
                    else:
                        return "Warning"
            return "Unknown"
            
        except Exception as e:
            logger.error(f"Error checking disk health: {e}")
            return "Error"
    
    def _get_drive_type(self, type_id: int) -> str:
        """
        Convert drive type ID to string.
        
        Args:
            type_id: Drive type ID
            
        Returns:
            str: Drive type description
        """
        drive_types = {
            0: "Unknown",
            1: "No Root Directory",
            2: "Removable Disk",
            3: "Local Disk",
            4: "Network Drive",
            5: "Compact Disc",
            6: "RAM Disk"
        }
        return drive_types.get(type_id, "Unknown")
    
    def _calculate_usage_percent(self, total: int, free: int) -> float:
        """
        Calculate disk usage percentage.
        
        Args:
            total: Total disk space
            free: Free disk space
            
        Returns:
            float: Usage percentage
        """
        if total and total > 0:
            used = total - free
            return round((used / total) * 100, 1)
        return 0.0
    
    def _format_size(self, size: int) -> str:
        """
        Format size in bytes to human-readable format.
        
        Args:
            size: Size in bytes
            
        Returns:
            str: Formatted size string
        """
        for unit in ['B', 'KB', 'MB', 'GB', 'TB']:
            if size < 1024:
                return f"{size:.1f} {unit}"
            size /= 1024
        return f"{size:.1f} PB"

def format_disk_info(info: Union[Dict, List[Dict]], format_type: str = "table") -> str:
    """
    Format disk information for display.
    
    Args:
        info: Disk information dictionary or list
        format_type: Output format ("table" or "json")
        
    Returns:
        str: Formatted information
    """
    if format_type == "json":
        return json.dumps(info, indent=2)
    
    if isinstance(info, list):
        if not info:
            return "No information available"
            
        if "name" in info[0]:  # Disk drives
            headers = ["Name", "Model", "Size", "Partitions", "Health"]
            data = [
                [
                    d["name"],
                    d["model"],
                    d["size"],
                    d["partitions"],
                    d["health"]
                ]
                for d in info
            ]
        else:  # Volumes
            headers = ["Drive", "Type", "Filesystem", "Size", "Free", "Usage %"]
            data = [
                [
                    v["name"],
                    v["drive_type"],
                    v["filesystem"],
                    v["size"],
                    v["free_space"],
                    f"{v['usage_percent']}%"
                ]
                for v in info
            ]
            
        return tabulate(data, headers=headers, tablefmt="grid")
        
    else:  # Single disk usage info
        if "error" in info:
            return f"Error: {info['error']}"
            
        data = [
            ["Path", info["path"]],
            ["Total Space", info["total"]],
            ["Used Space", info["used"]],
            ["Free Space", info["free"]],
            ["Usage", f"{info['usage_percent']}%"]
        ]
        
        return tabulate(data, headers=["Property", "Value"], tablefmt="grid")

def main():
    """Main function to handle command-line interface."""
    import argparse
    
    parser = argparse.ArgumentParser(description="Windows Disk Manager")
    parser.add_argument("--drives", action="store_true",
                      help="List physical disk drives")
    parser.add_argument("--volumes", action="store_true",
                      help="List disk volumes")
    parser.add_argument("--usage",
                      help="Get disk usage for specified path")
    parser.add_argument("--check",
                      help="Run disk check on specified drive")
    parser.add_argument("--cleanup",
                      help="Run disk cleanup on specified drive")
    parser.add_argument("--monitor",
                      help="Monitor disk usage for specified path")
    parser.add_argument("--interval", type=int, default=60,
                      help="Monitoring interval in seconds")
    parser.add_argument("--threshold", type=float, default=90.0,
                      help="Usage threshold percentage for alerts")
    parser.add_argument("--format", choices=["table", "json"], default="table",
                      help="Output format (table or json)")
    
    args = parser.parse_args()
    
    disk_manager = DiskManager()
    
    try:
        if args.drives:
            drives = disk_manager.get_disk_drives()
            print(format_disk_info(drives, args.format))
            
        elif args.volumes:
            volumes = disk_manager.get_volumes()
            print(format_disk_info(volumes, args.format))
            
        elif args.usage:
            usage = disk_manager.get_disk_usage(args.usage)
            print(format_disk_info(usage, args.format))
            
        elif args.check:
            result = disk_manager.check_disk(args.check)
            if args.format == "json":
                print(json.dumps(result, indent=2))
            else:
                if result["success"]:
                    print(f"Disk check completed successfully for {args.check}")
                    if result["output"]:
                        print("\nOutput:")
                        print(result["output"])
                else:
                    print(f"Disk check failed for {args.check}")
                    if result["error"]:
                        print(f"\nError: {result['error']}")
                        
        elif args.cleanup:
            result = disk_manager.cleanup_disk(args.cleanup)
            if args.format == "json":
                print(json.dumps(result, indent=2))
            else:
                if result["success"]:
                    print(f"Disk cleanup completed successfully for {args.cleanup}")
                    if result["output"]:
                        print("\nOutput:")
                        print(result["output"])
                else:
                    print(f"Disk cleanup failed for {args.cleanup}")
                    if result["error"]:
                        print(f"\nError: {result['error']}")
                        
        elif args.monitor:
            disk_manager.monitor_disk_usage(
                args.monitor,
                args.interval,
                args.threshold
            )
            
        else:
            parser.print_help()
            
    except Exception as e:
        logger.error(f"An error occurred: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main()
